/*
 * $Header$
 * $Revision: 2894 $
 * $Date: 2009-01-28 07:25:13 +1000 (Wed, 28 Jan 2009) $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003-2009 The JRDF Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        the JRDF Project (http://jrdf.sf.net/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The JRDF Project" and "JRDF" must not be used to endorse
 *    or promote products derived from this software without prior written
 *    permission. For written permission, please contact
 *    newmana@users.sourceforge.net.
 *
 * 5. Products derived from this software may not be called "JRDF"
 *    nor may "JRDF" appear in their names without prior written
 *    permission of the JRDF Project.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the JRDF Project.  For more
 * information on JRDF, please see <http://jrdf.sourceforge.net/>.
 *
 */

package org.jrdf.graph;

import java.net.URI;

public interface GraphValueFactory {
    /**
     * Create a URI reference.  Checks the validity of the URI (must be absolute).
     *
     * @param uri The URI of the resource.
     * @return the newly created URI reference value.
     * @throws org.jrdf.graph.GraphElementFactoryException If the resource failed to be created.
     */
    URIReference createURIReference(URI uri) throws GraphElementFactoryException;

    /**
     * Create a URI reference without checking if the URI given is a valid RDF
     * URI, currently if the URI is absolute.
     *
     * @param uri The URI of the resource.
     * @param validate true if we disbale checking to see if the URI is valid.
     * @return The newly created URI reference value.
     * @throws org.jrdf.graph.GraphElementFactoryException If the resource failed to be created.
     */
    URIReference createURIReference(URI uri, boolean validate) throws GraphElementFactoryException;

    /**
     * Converts a known Java class (such as Integer) into an RDF literal (XSD:int).
     *
     * @param object the value object (a Java class) to create the literal from.
     * @return the newly created literal value.
     * @throws org.jrdf.graph.GraphElementFactoryException If the literal failed to be created.
     * @throws IllegalArgumentException if there is no registered class to value creator mapping in the datatype
     *   factory.
     */
    Literal createLiteral(Object object) throws GraphElementFactoryException;

    /**
     * Creates a new literal with the given lexical value, with no language or
     * datatype.
     *
     * @param lexicalValue The lexical value for the literal.
     * @return the newly created literal value.
     * @throws org.jrdf.graph.GraphElementFactoryException If the literal failed to be created.
     */
    Literal createLiteral(String lexicalValue) throws GraphElementFactoryException;

    /**
     * Creates a new literal with the given lexical value, with a given language
     * but no datatype.
     *
     * @param lexicalValue The lexical value for the literal.  Cannot be null.
     * @param languageType The language of the literal or null if not required.
     * @return the newly created literal value.
     * @throws org.jrdf.graph.GraphElementFactoryException If the literal failed to be created.
     */
    Literal createLiteral(String lexicalValue, String languageType) throws GraphElementFactoryException;

    /**
     * Creates a new literal with the given lexical value and given datatype.
     *
     * @param lexicalValue The lexical value for the literal.  Cannot be null.
     * @param datatypeURI  The URI of the datatype of the literal or null if not
     *                     required.
     * @return the newly created literal value.
     * @throws org.jrdf.graph.GraphElementFactoryException If the literal failed to be created.
     */
    Literal createLiteral(String lexicalValue, URI datatypeURI) throws GraphElementFactoryException;
}