/*
 * $Header$
 * $Revision: 982 $
 * $Date: 2006-12-08 18:42:51 +1000 (Fri, 08 Dec 2006) $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003-2009 The JRDF Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        the JRDF Project (http://jrdf.sf.net/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The JRDF Project" and "JRDF" must not be used to endorse
 *    or promote products derived from this software without prior written
 *    permission. For written permission, please contact
 *    newmana@users.sourceforge.net.
 *
 * 5. Products derived from this software may not be called "JRDF"
 *    nor may "JRDF" appear in their names without prior written
 *    permission of the JRDF Project.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the JRDF Project.  For more
 * information on JRDF, please see <http://jrdf.sourceforge.net/>.
 *
 */

package org.jrdf.graph.global.molecule;

import org.jrdf.graph.Triple;

import java.util.Set;

/**
 * These methods are called as the {@link MoleculeTraverser} traverses a molecule.  For a given molecule:
 * _1 p _2, _1 p _3, _2 p o.  First handleTriple is called with both the first triple (_1 p _2).  Next,
 * handleStartContainsMolecules is called which has the set of molecules (one molecule that contains _1 p _3).  Then,
 * handleTriple is called with _1 p _3.  Then handleEndContainsMolecules is called with the same set of molecules.  And
 * so on...
 *
 * @author Andrew Newman
 * @version $Id: MoleculeHandler.java 2894 2009-01-27 21:25:13Z newmana $
 */
public interface MoleculeHandler {
    /**
     * A triple in the root set of triples is found.
     *
     * @param triple the triple found.
     */
    void handleTriple(Triple triple);

    /**
     * The is called instead of handleStart/End contains molecules when the triple visited does not contain any
     * submolecules.
     */
    void handleEmptyMolecules();

    /**
     * This is called when the current top level triple contains a set of submolecules.
     *
     * @param newMolecules the submolecules belonging to the top level triple.
     */
    void handleStartContainsMolecules(Set<Molecule> newMolecules);

    /**
     * This is called when the top level triple's submolecules have all been visited.
     *
     * @param newMolecules the submolecules belonging to the top level triple.
     */
    void handleEndContainsMolecules(Set<Molecule> newMolecules);
}
