/*
 * $Header$
 * $Revision: 3197 $
 * $Date: 2009-11-14 06:20:58 +1000 (Sat, 14 Nov 2009) $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003-2009 The JRDF Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        the JRDF Project (http://jrdf.sf.net/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The JRDF Project" and "JRDF" must not be used to endorse
 *    or promote products derived from this software without prior written
 *    permission. For written permission, please contact
 *    newmana@users.sourceforge.net.
 *
 * 5. Products derived from this software may not be called "JRDF"
 *    nor may "JRDF" appear in their names without prior written
 *    permission of the JRDF Project.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the JRDF Project.  For more
 * information on JRDF, please see <http://jrdf.sourceforge.net/>.
 *
 */

package org.jrdf.query.answer;

import static org.jrdf.query.answer.SparqlResultType.BOOLEAN;
import org.jrdf.util.ClosableIterator;
import org.jrdf.util.ClosableIteratorImpl;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Yuan-Fang Li
 * @version $Id: AskAnswerImpl.java 3197 2009-11-13 20:20:58Z newmana $
 */
public class AskAnswerImpl implements AskAnswer, Serializable {
    private static final String[] NO_VARIABLES = new String[]{};
    private static final long serialVersionUID = 432026021050798815L;
    private long timeTaken;
    private boolean result;
    private static final int ONE_RESULT = 1;

    private AskAnswerImpl() {
    }

    public AskAnswerImpl(long newTimeTaken, boolean newResult) {
        this.timeTaken = newTimeTaken;
        this.result = newResult;
    }

    public long getTimeTaken() {
        return timeTaken;
    }

    public boolean getResult() {
        return result;
    }

    public long numberOfTuples() {
        return ONE_RESULT;
    }

    public String[] getVariableNames() {
        return NO_VARIABLES;
    }

    public ClosableIterator<TypeValue[]> columnValuesIterator() {
        TypeValue typeValue = new TypeValueImpl(BOOLEAN, Boolean.toString(result));
        Set<TypeValue[]> set = new HashSet<TypeValue[]>();
        set.add(new TypeValue[]{typeValue});
        return new ClosableIteratorImpl<TypeValue[]>(set.iterator());
    }

    public String toString() {
        return "Boolean Result" + "\nValue " + result;
    }

    public <R> R accept(AnswerVisitor<R> visitor) {
        return visitor.visitAskAnswer(this);
    }
}
