/*
 * $Header$
 * $Revision: 3132 $
 * $Date: 2009-10-04 12:20:14 +1000 (Sun, 04 Oct 2009) $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003-2009 The JRDF Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        the JRDF Project (http://jrdf.sf.net/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The JRDF Project" and "JRDF" must not be used to endorse
 *    or promote products derived from this software without prior written
 *    permission. For written permission, please contact
 *    newmana@users.sourceforge.net.
 *
 * 5. Products derived from this software may not be called "JRDF"
 *    nor may "JRDF" appear in their names without prior written
 *    permission of the JRDF Project.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the JRDF Project.  For more
 * information on JRDF, please see <http://jrdf.sourceforge.net/>.
 *
 */

package org.jrdf.query.relation.operation.mem.join;

import org.jrdf.query.relation.EvaluatedRelation;
import static org.jrdf.query.relation.constants.RelationDEE.RELATION_DEE;
import static org.jrdf.query.relation.constants.RelationDUM.RELATION_DUM;
import org.jrdf.query.relation.operation.NadicJoin;
import org.jrdf.query.relation.operation.mem.common.RelationProcessor;

import java.util.Collections;
import java.util.Set;

/**
 * A simple memory based implementation of NadicJoin.
 *
 * @author Andrew Newman
 * @version $Revision: 3132 $
 */
public final class NadicJoinImpl implements NadicJoin {
    private final TupleEngine tupleEngine;
    private final RelationProcessor relationProcessor;

    public NadicJoinImpl(RelationProcessor newRelationProcessor, TupleEngine newTupleEngine) {
        this.tupleEngine = newTupleEngine;
        this.relationProcessor = newRelationProcessor;
    }

    public EvaluatedRelation join(Set<EvaluatedRelation> relations) {
        EvaluatedRelation relation = isDeeDumOrSingle(relations);
        if (relation == null) {
            relation = relationProcessor.processRelations(relations, tupleEngine);
        }
        return relation;
    }

    private EvaluatedRelation isDeeDumOrSingle(Set<EvaluatedRelation> relations) {
        EvaluatedRelation relation = null;
        // Is it the empty collection return DEE.
        if (relations.equals(Collections.<EvaluatedRelation>emptySet())) {
            relation = RELATION_DEE;
        }
        // Is it DUM - therefore return DUM
        if (relations.contains(RELATION_DUM)) {
            relation = RELATION_DUM;
        }
        // If relations contains DEE, remove it and return the rest of the set.
        if (relations.contains(RELATION_DEE) && relations.size() > 1) {
            relations.remove(RELATION_DEE);
        }
        // Is it just one relations - if so just return it back.
        if (relations.size() == 1) {
            relation = relations.iterator().next();
        }
        return relation;
    }
}
